

def install_model_depend(model_name:str, **kwargs):
    is_inference = kwargs.get('is_inference',False)
    inference_wrapper = kwargs.get('inference_wrapper',False)
    use_vllm = kwargs.get('use_vllm',False)
    use_galore = kwargs.get('use_galore',False)
    finetune_type = kwargs.get('finetune_type','lora')
    import subprocess
    from ailab.atp_finetuner.constant import Model
    if use_vllm:
        if model_name == Model.deepseek_v2_chat:
            subprocess.check_call("pip install vllm==1.4.7 -i https://repo.model.xfyun.cn/api/packages/administrator/pypi/simple --extra-index-url https://pypi.mirrors.ustc.edu.cn/simple/", shell=True)
        else:
            subprocess.check_call("pip install vllm -i https://repo.model.xfyun.cn/api/packages/administrator/pypi/simple --extra-index-url https://pypi.mirrors.ustc.edu.cn/simple/", shell=True)
        if (model_name == Model.gemma2_9b_it or model_name == Model.gemma2_27b_it):
            import os
            os.environ['VLLM_ATTENTION_BACKEND'] = 'FLASHINFER'
        return
    if model_name == Model.baichuan_13b:
        subprocess.check_call("pip install transformers_stream_generator", shell=True)
    if model_name == Model.chinese_alpaca:
        subprocess.check_call("pip install transformers==4.29.1", shell=True)
    if model_name in [Model.internlm_7b]:
        subprocess.check_call("pip install transformers==4.31.0", shell=True)
    if model_name in [Model.starcoder2_3b, Model.starcoder2_3b_instruct, 
                      Model.starcoder2_7b, Model.starcoder2_7b_instruct,
                      Model.gemma_7b_it,Model.qwen_v2_0b5_chat,Model.qwen_v2_1b5_chat,Model.qwen_v2_7b_chat]:
        subprocess.check_call("pip install transformers==4.39.0 trl==0.8.1 accelerate==0.27.2", shell=True)
    if model_name == Model.falcon_7b or Model.falcon_7b_instruct:
        subprocess.check_call("pip install einops", shell=True)
    if model_name in [Model.deepseek_v2_chat]:
        subprocess.check_call("pip install transformers==4.39.3", shell=True)
    if model_name in [Model.yolos_base, Model.yolos_small]:
        subprocess.check_call("pip install albumentations pycocotools", shell=True)
    if use_galore:
        if finetune_type != 'full':
            raise ValueError('if use galore, finetune_type should be full')
        subprocess.check_call("pip install galore-torch==1.0", shell=True)
    if model_name in [Model.stable_diffusion_1_4,Model.stable_diffusion_2_1]:
        if is_inference or inference_wrapper:
            subprocess.check_call("pip install peft==0.6.0 diffusers transformers==4.34.0", shell=True)
        else:
            subprocess.check_call("pip install diffusers transformers==4.31.0", shell=True)
    if model_name in [Model.stable_diffusion_xl_base_1,Model.stable_diffusion_3_medium]:
        if is_inference or inference_wrapper:
            subprocess.check_call("pip install opencv-python piexif", shell=True)
        else:
            subprocess.check_call("pip install xformers==0.0.22", shell=True)
    if model_name in [Model.stable_diffusion_3_medium]:
        subprocess.check_call("pip install diffusers==0.29.1 transformers==4.41.0", shell=True)
    if model_name in [Model.meetkai_func_small_v2_5]:
        subprocess.check_call("pip install transformers==4.41.2 pydantic==2.6.4", shell=True)
    if model_name in [Model.glm_4v_9b]:
        subprocess.check_call("pip install tiktoken==0.7.0", shell=True)
    if model_name in [Model.minicpm_llama3_v_2_5]:
        subprocess.check_call("pip install transformers==4.41.2", shell=True)
    if model_name in [Model.bge_reranker_v2_m3,Model.bge_visualized_m3]:
        subprocess.check_call("pip install FlagEmbedding -U", shell=True)
    if model_name in [Model.bge_visualized_m3]:
        subprocess.check_call("pip install torchvision timm einops ftfy", shell=True)

def get_template_name(model_name:str):
    from ailab.atp_finetuner.constant import Model
    template_dict = {
            Model.falcon_7b_instruct : "falcon",
            Model.moss_moon_003_base : "moss",
            Model.llama2_7b_chat_hf : "llama2",
            Model.llama2_13b_chat_hf : "llama2",
            Model.internlm2_7b_chat : "intern2",
            Model.xverse_13b : "vanilla",
            Model.lawgpt_llama : "alpaca",
            Model.educhat : "edu",
            Model.codellama_7b_instruction : "llama2",
            Model.codellama_13b_instruction : "llama2",
            Model.atom_7b : "atom",
            Model.chatglm3_6b : "chatglm3",
            Model.mixtral_8x7B_v01 : "mistral",
            Model.chinese_mixtral_instruct : "mistral",
            Model.llama3_8b_instruct : "llama3",
            Model.llama3_cn_8b_instruct : "llama3",
            Model.chinese_alpaca_2 : "llama2_zh",
            Model.chinese_alpaca_2_13b : "llama2_zh",
            Model.chinese_alpaca_2_7b_16k : "llama2_zh",
            Model.chinese_alpaca_2_13b_16k : "llama2_zh",
            Model.chinese_alpaca_2_1b3 : "llama2_zh",
            Model.phi_3_mini_128k_instruct : "phi",
            Model.phi_3_mini_4k_instruct : "phi",
            Model.gemma_7b_it : "gemma",
            Model.llava_15_7b_hf : "vicuna",
            Model.qwen_v2_7b_chat : "qwen",
            Model.qwen_v2_1b5_chat : "qwen",
            Model.qwen_v2_0b5_chat : "qwen",
            Model.meetkai_func_small_v2_5 : "llama3",
        }
    return template_dict.get(model_name,"default")

from packaging import version
from packaging.version import Version
import importlib.metadata
import importlib.util
def _is_package_available(name: str) -> bool:
    return importlib.util.find_spec(name) is not None

def _get_package_version(name: str) -> "Version":
    try:
        return version.parse(importlib.metadata.version(name))
    except Exception:
        return version.parse("0.0.0")
    
def is_fa2_enable(model_name:str):
    from ailab.atp_finetuner.constant import Model
    if model_name in [Model.baichuan_7b,Model.baichuan_13b,Model.bloomz_7b1_mt,
                      Model.bloomz_3b,Model.bloomz_1b1,Model.falcon_7b,Model.falcon_7b_instruct,
                    Model.belle_7b_2m,Model.chatglm3_6b,Model.internlm2_7b_chat,Model.internlm_7b]:
        return False
    fa2_enable = _is_package_available("flash_attn") and _get_package_version("flash_attn") > version.parse("2.0.0")
    transformers_enable = _get_package_version("transformers") >= version.parse("4.36.0")
    print(f'fa2_enable {fa2_enable},transformers_enable {transformers_enable}')
    return fa2_enable and transformers_enable